import java.awt.Graphics2D;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.Window;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.*;
import javax.imageio.ImageIO;

/** Need to not create configuration panel unless it is opened. */
public class Yes {
	private static Thread _loop;
	private YesClock _clock;
	private Window _frame;
	private ConfigurationPanel _configuration_panel;
	private HandPanel _hand_config;
	private Configuration _configuration;
	
	public static final void main( String[] args ) {
		boolean frame = true;
		if (args.length != 0) {
			List arg_list = Arrays.asList( args );
			
			if (arg_list.contains("--noframe")) frame = false;

			int sx = 500;
			int sy = 500;
			
			if (arg_list.contains("--size")) {
				int ind = arg_list.indexOf("--size");
				String x = (String)arg_list.get( ind+1 );
				String y = (String)arg_list.get( ind+2 );
				sx = Integer.valueOf(x).intValue();
				sy = Integer.valueOf(y).intValue();
			}

			if (arg_list.contains("--dump")) {
				int name_ind = arg_list.indexOf("--dump")+1;
				String name = (String)arg_list.get(name_ind);
				// dump
				BufferedImage i = new BufferedImage( sx, sy, BufferedImage.TYPE_INT_RGB);
				Graphics2D g = i.createGraphics();

				YesClock yesclock = new YesClock();
				Properties config = new Configuration();
				updateBasicConfig( config, yesclock );
				yesclock.setSize( sx, sy );
				yesclock.redraw();
				yesclock.paint( g );
				File f = new File( name );
				try { ImageIO.write( i, "png", f ); }
				catch (IOException ioe) { ioe.printStackTrace(); }
				// write image here
				return;
			}
		}

		Yes y = new Yes();
		y.initialize( frame );
		y.getWindow().setVisible( true );
		y.start();
	}

	public Yes(){}

	public void initialize( boolean frame ) {
		_clock = new YesClock();
		_configuration = new Configuration();
		
		if (frame) _frame = new JFrame("YES");
		else _frame = new JWindow();

		RootPaneContainer container = (RootPaneContainer)_frame;

		_frame.addWindowListener( new WindowAdapter() {
			public void windowClosing( WindowEvent we ) { exit(); }
		});

		container.getContentPane().setLayout( new BorderLayout() );
		container.getContentPane().add( BorderLayout.CENTER, _clock );

		JFrame f = null;
		if (frame) f = (JFrame)_frame;
		//_configuration_panel = new ConfigurationPanel( f );
		_hand_config = new HandPanel( f, this );
		updateConfiguration();

		final JPopupMenu menu = makeMenu( );
		//menu.setLightWeightPopupEnabled(false);
		_clock.addMouseListener( new MouseAdapter() {
			public void mouseReleased( MouseEvent me ) {show(me);}
			public void mousePressed( MouseEvent me ) {show(me);}
			public void show( MouseEvent me ) {
				if (me.isPopupTrigger()) {
					menu.show( me.getComponent(), me.getX(), me.getY() );
				}
			}
		});

		int x = Integer.valueOf(_configuration.getProperty("x_loc", "100")).intValue();
		int y = Integer.valueOf(_configuration.getProperty("y_loc", "80")).intValue();
		int w = Integer.valueOf(_configuration.getProperty("width", "400")).intValue();
		int h = Integer.valueOf(_configuration.getProperty("height", "400")).intValue();
		_frame.setBounds(x,y,w,h);

		// Do this last to avoid triggering a save event
		_frame.addComponentListener( new ComponentAdapter() {
			public void componentMoved( ComponentEvent ae ) {
				//System.out.println(ae.paramString());
				move( ae.getComponent().getBounds() );
			}
			public void componentResized( ComponentEvent ae ) {
				//System.out.println(ae.paramString());
				resize( ae.getComponent().getBounds() );
			}
		});

	}

	private void exit() {
		_loop.interrupt();
		if (_frame != null) _frame.dispose();
		System.exit(0);
	}

	protected Window getWindow() { return _frame; }
	
	protected void start() {
		_loop = new Thread( _clock );
		_loop.start();
	}
	
	private JPopupMenu makeMenu( ) {
		JPopupMenu menu = new JPopupMenu();
		JMenuItem item = menu.add( "Configuration..." );
		final Yes stupid_java = this;
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) { 		
				if (_configuration_panel == null)
					_configuration_panel = new ConfigurationPanel( 
						(_frame instanceof JFrame ? (JFrame)_frame : null), 
						_configuration );
				_configuration_panel.expose(stupid_java);
			 } 
		});

		String def = "Change to EDT";
		if (_clock.getEDT()) def = "Change to standard";
		final JMenuItem time = menu.add( def );
		time.addActionListener( new ActionListener() {
			private String edt = "Change to EDT",
				standard = "Change to standard";
			private boolean edt_on = _clock.getEDT();
			public void actionPerformed( ActionEvent ae  ) {
				// Change the label of this menu item
				if (edt_on) {
					time.setText( edt );
					edt_on = false;
				} else {
					time.setText( standard );
					edt_on = true;
				}
				// Change the Clock to whatever it needs to be
				_clock.setEDT( edt_on );
				// Update the configuration
				_configuration.setProperty("edt_on", String.valueOf(edt_on));
				_configuration.save();
				// Redraw and repaint the clock.
				_clock.redraw();
				_clock.repaint();
			}
		});

		item = menu.add( "Change hand shape..." );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) {
				_hand_config.setVisible(true);
			}
		});

		item = menu.add( "About..." );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent ae ) { about(); } 
		});

		menu.addSeparator();

		item = menu.add( "Exit" );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) { exit(); }
		});
		return menu;
	}

	protected void setHand( String hand_name ) {
		_configuration.setProperty( "hand_name", hand_name );
		_configuration.save();
		updateConfiguration();
	}

	protected static void updateBasicConfig( Properties config, YesClock clock ) {
		// Get all of the preferences as Strings
		String s_lon = config.getProperty("longitude", "-121.31104");
		String s_lat = config.getProperty( "latitude", "44.06604" );
		String s_edt = config.getProperty("edt_on", "false");
		String s_tz = config.getProperty( "timezone", String.valueOf(
					(int)(Calendar.getInstance().get(Calendar.ZONE_OFFSET)/3600000)
				));
		// Set them back to the configuration
		config.setProperty( "longitude", s_lon);
		config.setProperty( "latitude", s_lat);
		config.setProperty( "timezone", s_tz);
		config.setProperty( "edt_on", s_edt);
		// Turn them all into their true value types
		float lon = Float.valueOf( s_lon ).floatValue();
		float lat = Float.valueOf( s_lat ).floatValue();
		boolean edt_on = Boolean.valueOf( s_edt ).booleanValue();
		int tz = Integer.valueOf( s_tz ).intValue();
		// Now set the clock
		clock.setLongitude( lon );
		clock.setLatitude( lat );
		clock.setEDT( edt_on );
		clock.setTZ( tz );
	}

	protected void updateConfiguration() {
		Properties config = _configuration;
		updateBasicConfig( config, _clock );
		String hand_name = config.getProperty( "hand_name" );
		if (hand_name != null) {
			Hand hand = _hand_config.getHand( hand_name );
			_clock.setHandLine( hand.getStroke() );
			_clock.setHandFill( hand.getFill() );
			_clock.setHand( hand.getShape() );
		} 
		String color = config.getProperty( "day" );
		if(color!=null) _clock.setDay( Color.decode( color ) );
		color = config.getProperty( "night" );
		if(color!=null) _clock.setNight( Color.decode( color ) );
		color = config.getProperty( "background" );
		if(color!=null) _clock.setBackground( Color.decode( color ) );

		String bool = config.getProperty( "markers" );
		if(bool!=null && bool.equals("false")) _clock.showMarkers(false);
		else _clock.showMarkers(true);
		bool = config.getProperty( "time" );
		if(bool!=null && bool.equals("false")) _clock.showDigitalTime(false);
		else _clock.showDigitalTime(true);
		bool = config.getProperty( "jewels" );
		if(bool!=null && bool.equals("false")) _clock.showJewels(false);
		else _clock.showJewels(true);
		bool = config.getProperty( "24hr" );
		if(bool!=null && bool.equals("false")) _clock.showJewels(false);
		else _clock.show24HR(true);

		_clock.redraw();
		_clock.repaint();
	}

	private void move( Rectangle bounds ) { 
		_configuration.setProperty( "x_loc", String.valueOf(bounds.x) );
		_configuration.setProperty( "y_loc", String.valueOf(bounds.y) );
		_configuration.save();
	}
	private void resize( Rectangle bounds ) {
		_configuration.setProperty( "width", String.valueOf(bounds.width) );
		_configuration.setProperty( "height", String.valueOf(bounds.height) );
		_configuration.save();
	}
	
	private void about() {
		String version = "1.0";
		JFrame f = null;
		if (_frame instanceof JFrame) f = (JFrame)_frame;
		JOptionPane.showMessageDialog( f,
		"<HTML><H1>YesClock</H1>v"+Version.VERSION+"<BR>"+Version.DATE+"<P>"+
		Version.COPYRIGHT+"<BR>"+
		"http://www.germane-software.com/~ser<P>"+
		"For a real Yes watch, visit:<BR>http://www.yeswatch.com</HTML>",
		"About YesClock",
		JOptionPane.PLAIN_MESSAGE );
	}
}
